@@ -41,9 +41,9 @@
  */
 
 /* To compile a test program, do:
- * $ gcc -DSYSTZ_GET_TEST -Wall -o system-timezone-get `pkg-config --cflags --libs glib-2.0 gobject-2.0 gio-2.0` system-timezone.c
+ * $ gcc -DSYSTZ_GET_TEST -Wall -g -O0 -o system-timezone-get `pkg-config --cflags --libs glib-2.0 gobject-2.0 gio-2.0` system-timezone.c
  * or:
- * $ gcc -DSYSTZ_SET_TEST -Wall -o system-timezone-set `pkg-config --cflags --libs glib-2.0 gobject-2.0 gio-2.0` system-timezone.c
+ * $ gcc -DSYSTZ_SET_TEST -Wall -g -O0 -o system-timezone-set `pkg-config --cflags --libs glib-2.0 gobject-2.0 gio-2.0` system-timezone.c
  *
  * You can then read the system timezone with:
  * $ system-timezone-get
@@ -53,7 +53,9 @@
 
 
 /* FIXME: it'd be nice to filter out the timezones that we might get when
- * parsing config files that are not in zone.tab */
+ * parsing config files that are not in zone.tab. Note that it's also wrong
+ * in some cases: eg, in tzdata2008b, Asia/Calcutta got renamed to
+ * Asia/Kolkata and the old name is not in zone.tab. */
 
 #include <string.h>
 #include <unistd.h>
@@ -69,6 +71,7 @@
 #ifndef SYSTZ_SET_TEST
 #define ETC_TIMEZONE        "/etc/timezone"
 #define ETC_TIMEZONE_MAJ    "/etc/TIMEZONE"
+#define ETC_RC_CONF         "/etc/rc.conf"
 #define ETC_SYSCONFIG_CLOCK "/etc/sysconfig/clock"
 #define ETC_CONF_D_CLOCK    "/etc/conf.d/clock"
 #define ETC_LOCALTIME       "/etc/localtime"
@@ -77,6 +80,7 @@
 #define TEST_PREFIX         "/tmp/systz-test"
 #define ETC_TIMEZONE        TEST_PREFIX"/etc/timezone"
 #define ETC_TIMEZONE_MAJ    TEST_PREFIX"/etc/TIMEZONE"
+#define ETC_RC_CONF         TEST_PREFIX"/etc/rc.conf"
 #define ETC_SYSCONFIG_CLOCK TEST_PREFIX"/etc/sysconfig/clock"
 #define ETC_CONF_D_CLOCK    TEST_PREFIX"/etc/conf.d/clock"
 #define ETC_LOCALTIME       TEST_PREFIX"/etc/localtime"
@@ -303,14 +307,14 @@ system_timezone_monitor_changed (GFileMonitor *handle,
  *  + However, most distributions also have the timezone setting
  *    configured somewhere else. This might be better to read it from there.
  *
+ *    Debian/Ubuntu/Gentoo (new): content of /etc/timezone
  *    Fedora/Mandriva: the ZONE key in /etc/sysconfig/clock
  *    openSUSE: the TIMEZONE key in /etc/sysconfig/clock
- *    Gentoo (old): the ZONE key in /etc/conf.d/clock
- *    Debian/Ubuntu/Gentoo (new): content of /etc/timezone
  *    Solaris/OpenSolaris: the TZ key in /etc/TIMEZONE
+ *    Arch Linux: the TIMEZONE key in /etc/rc.conf
+ *    Gentoo (old): the ZONE key in /etc/conf.d/clock
  *
  *    FIXME: reading the system-tools-backends, it seems there's this too:
- *           ArchLinux: the TIMEZONE key in /etc/rc.conf
  *           Solaris: the TZ key in /etc/default/init
  *                    /etc/TIMEZONE seems to be a link to /etc/default/init
  *
@@ -318,7 +322,7 @@ system_timezone_monitor_changed (GFileMonitor *handle,
  *
  */
 
-/* This works for Debian and derivatives (including Ubuntu) */
+/* This works for Debian and derivatives (including Ubuntu), and new Gentoo */
 static char *
 system_timezone_read_etc_timezone (void)
 {
@@ -443,14 +447,14 @@ system_timezone_write_key_file (const char  *filename,
                                 const char  *value,
                                 GError     **error)
 {
-        GError        *our_error;
-        char          *content;
-        unsigned int   len;
-        char          *key_eq;
-        char         **lines;
-        gboolean       replaced;
-        gboolean       retval;
-        int            n;
+        GError    *our_error;
+        char      *content;
+        gsize      len;
+        char      *key_eq;
+        char     **lines;
+        gboolean   replaced;
+        gboolean   retval;
+        int        n;
         
         if (!g_file_test (filename, G_FILE_TEST_IS_REGULAR))
                 return TRUE;
@@ -582,12 +586,50 @@ system_timezone_write_etc_conf_d_clock (const char  *tz,
                                                "TIMEZONE", tz, error);
 }
 
+/* This works for Arch Linux */
+static char *
+system_timezone_read_etc_rc_conf (void)
+{
+        return system_timezone_read_key_file (ETC_RC_CONF,
+                                              "TIMEZONE");
+}
+
+static gboolean
+system_timezone_write_etc_rc_conf (const char  *tz,
+                                   GError     **error)
+{
+        return system_timezone_write_key_file (ETC_RC_CONF,
+                                               "TIMEZONE", tz, error);
+}
+
 /*
  *
  * First, getting the timezone.
  *
  */
 
+static char *
+system_timezone_strip_path_if_valid (const char *filename)
+{
+        int skip;
+
+        if (!filename || !g_str_has_prefix (filename, SYSTEM_ZONEINFODIR"/"))
+                return NULL;
+
+        /* Timezone data files also live under posix/ and right/ for some
+         * reason.
+         * FIXME: make sure accepting those files is valid. I think "posix" is
+         * okay, not sure about "right" */
+        if (g_str_has_prefix (filename, SYSTEM_ZONEINFODIR"/posix/"))
+                skip = strlen (SYSTEM_ZONEINFODIR"/posix/");
+        else if (g_str_has_prefix (filename, SYSTEM_ZONEINFODIR"/right/"))
+                skip = strlen (SYSTEM_ZONEINFODIR"/right/");
+        else
+                skip = strlen (SYSTEM_ZONEINFODIR"/");
+
+        return g_strdup (filename + skip);
+}
+
 /* Read the soft symlink from /etc/localtime */
 static char *
 system_timezone_read_etc_localtime_softlink (void)
@@ -598,31 +640,23 @@ system_timezone_read_etc_localtime_softlink (void)
         if (!g_file_test (ETC_LOCALTIME, G_FILE_TEST_IS_SYMLINK))
                 return NULL;
 
-        tz = NULL;
-
         file = g_file_read_link (ETC_LOCALTIME, NULL);
-        if (g_str_has_prefix (file, SYSTEM_ZONEINFODIR"/"))
-                tz = g_strdup (file + strlen (SYSTEM_ZONEINFODIR"/"));
+        tz = system_timezone_strip_path_if_valid (file);
         g_free (file);
 
-        if (!tz || tz[0] == '\0') {
-                g_free (tz);
-                tz = NULL;
-        }
-
         return tz;
 }
 
-typedef gboolean (*CompareFiles) (struct stat  *a_stat,
-                                  struct stat  *b_stat,
-                                  const char   *a_content,
-                                  unsigned int  a_content_len,
-                                  const char   *b_filename);
+typedef gboolean (*CompareFiles) (struct stat *a_stat,
+                                  struct stat *b_stat,
+                                  const char  *a_content,
+                                  gsize        a_content_len,
+                                  const char  *b_filename);
 
 static char *
 recursive_compare (struct stat  *localtime_stat,
                    const char   *localtime_content,
-                   unsigned int  localtime_content_len,
+                   gsize         localtime_content_len,
                    char         *file,
                    CompareFiles  compare_func)
 {
@@ -637,7 +671,7 @@ recursive_compare (struct stat  *localtime_stat,
                                   localtime_content,
                                   localtime_content_len,
                                   file))
-                        return g_strdup (file + strlen (SYSTEM_ZONEINFODIR"/"));
+                        return system_timezone_strip_path_if_valid (file);
                 else
                         return NULL;
         } else if (S_ISDIR (file_stat.st_mode)) {
@@ -675,11 +709,11 @@ recursive_compare (struct stat  *localtime_stat,
 
 
 static gboolean
-files_are_identical_inode (struct stat  *a_stat,
-                           struct stat  *b_stat,
-                           const char   *a_content,
-                           unsigned int  a_content_len,
-                           const char   *b_filename)
+files_are_identical_inode (struct stat *a_stat,
+                           struct stat *b_stat,
+                           const char  *a_content,
+                           gsize        a_content_len,
+                           const char  *b_filename)
 {
         return (a_stat->st_ino == b_stat->st_ino);
 }
@@ -706,15 +740,15 @@ system_timezone_read_etc_localtime_hardlink (void)
 }
 
 static gboolean
-files_are_identical_content (struct stat  *a_stat,
-                             struct stat  *b_stat,
-                             const char   *a_content,
-                             unsigned int  a_content_len,
-                             const char   *b_filename)
+files_are_identical_content (struct stat *a_stat,
+                             struct stat *b_stat,
+                             const char  *a_content,
+                             gsize        a_content_len,
+                             const char  *b_filename)
 {
-        char         *b_content = NULL;
-        unsigned int  b_content_len = -1;
-        int           cmp;
+        char  *b_content = NULL;
+        gsize  b_content_len = -1;
+        int    cmp;
 
         if (a_stat->st_size != b_stat->st_size)
                 return FALSE;
@@ -738,9 +772,10 @@ files_are_identical_content (struct stat  *a_stat,
 static char *
 system_timezone_read_etc_localtime_content (void)
 {
-        struct stat   stat_localtime;
-        char         *localtime_content = NULL;
-        unsigned int  localtime_content_len = -1;
+        struct stat  stat_localtime;
+        char        *localtime_content = NULL;
+        gsize        localtime_content_len = -1;
+        char        *retval;
 
         if (g_stat (ETC_LOCALTIME, &stat_localtime) != 0)
                 return NULL;
@@ -754,21 +789,33 @@ system_timezone_read_etc_localtime_content (void)
                                   NULL))
                 return NULL;
 
-        return recursive_compare (&stat_localtime,
-                                  localtime_content,
-                                  localtime_content_len,
-                                  SYSTEM_ZONEINFODIR,
-                                  files_are_identical_content);
+        retval = recursive_compare (&stat_localtime,
+                                   localtime_content,
+                                   localtime_content_len,
+                                   SYSTEM_ZONEINFODIR,
+                                   files_are_identical_content);
+
+        g_free (localtime_content);
+
+        return retval;
 }
 
 typedef char * (*GetSystemTimezone) (void);
+/* The order of the functions here define the priority of the methods used
+ * to find the timezone. First method has higher priority. */
 static GetSystemTimezone get_system_timezone_methods[] = {
+        /* cheap and "more correct" than data from a config file */
+        system_timezone_read_etc_localtime_softlink,
+        /* reading various config files */
         system_timezone_read_etc_timezone,
-        system_timezone_read_etc_TIMEZONE,
         system_timezone_read_etc_sysconfig_clock,
         system_timezone_read_etc_sysconfig_clock_alt,
+        system_timezone_read_etc_TIMEZONE,
+        system_timezone_read_etc_rc_conf,
+        /* reading deprecated config files */
         system_timezone_read_etc_conf_d_clock,
-        system_timezone_read_etc_localtime_softlink,
+        /* reading /etc/timezone directly. Expensive since we have to stat
+         * many files */
         system_timezone_read_etc_localtime_hardlink,
         system_timezone_read_etc_localtime_content,
         NULL
@@ -819,11 +866,11 @@ static gboolean
 system_timezone_is_zone_file_valid (const char  *zone_file,
                                     GError     **error)
 {
-        GError       *our_error;
-        GIOChannel   *channel;
-        GIOStatus     status;
-        char          buffer[strlen (TZ_MAGIC)];
-        unsigned int  read;
+        GError     *our_error;
+        GIOChannel *channel;
+        GIOStatus   status;
+        char        buffer[strlen (TZ_MAGIC)];
+        gsize       read;
 
         /* First, check the zone_file is properly rooted */
         if (!g_str_has_prefix (zone_file, SYSTEM_ZONEINFODIR"/")) {
@@ -863,7 +910,7 @@ system_timezone_is_zone_file_valid (const char  *zone_file,
                 return FALSE;
         }
 
-        if (read != 4 || strcmp (buffer, TZ_MAGIC) != 0) {
+        if (read != strlen (TZ_MAGIC) || strncmp (buffer, TZ_MAGIC, strlen (TZ_MAGIC)) != 0) {
                 g_set_error (error, SYSTEM_TIMEZONE_ERROR,
                              SYSTEM_TIMEZONE_ERROR_INVALID_TIMEZONE_FILE,
                              "%s is not a timezone file",
@@ -878,9 +925,9 @@ static gboolean
 system_timezone_set_etc_timezone (const char  *zone_file,
                                   GError     **error)
 {
-        GError       *our_error;
-        char         *content;
-        unsigned int  len;
+        GError *our_error;
+        char   *content;
+        gsize   len;
 
         if (!system_timezone_is_zone_file_valid (zone_file, error))
                 return FALSE;
@@ -924,11 +971,20 @@ system_timezone_set_etc_timezone (const char  *zone_file,
 
 typedef gboolean (*SetSystemTimezone) (const char  *tz,
                                        GError     **error);
+/* The order here does not matter too much: we'll try to change all files
+ * that already have a timezone configured. It matters in case of error,
+ * since the process will be stopped and the last methods won't be called.
+ * So we use the same order as in get_system_timezone_methods */
 static SetSystemTimezone set_system_timezone_methods[] = {
+        /* writing various config files if they exist and have the
+         * setting already present */
         system_timezone_write_etc_timezone,
-        system_timezone_write_etc_TIMEZONE,
         system_timezone_write_etc_sysconfig_clock,
         system_timezone_write_etc_sysconfig_clock_alt,
+        system_timezone_write_etc_TIMEZONE,
+        system_timezone_write_etc_rc_conf,
+        /* writing deprecated config files if they exist and have the
+         * setting already present */
         system_timezone_write_etc_conf_d_clock,
         NULL
 };
