@@ -27,7 +27,7 @@
 
 #include <libbonobo.h>
 
-#include "menu.h"
+#include <gmenu-tree.h>
 
 #include "launcher.h"
 #include "panel.h"
@@ -43,6 +43,7 @@
 #include "panel-util.h"
 #include "panel-profile.h"
 #include "panel-addto.h"
+#include "panel-icon-names.h"
 
 typedef struct {
 	PanelWidget *panel_widget;
@@ -109,7 +110,7 @@ static PanelAddtoItemInfo special_addto_items [] = {
 	{ PANEL_ADDTO_LAUNCHER_NEW,
 	  N_("Custom Application Launcher"),
 	  N_("Create a new launcher"),
-	  "launcher-program",
+	  PANEL_ICON_LAUNCHER,
 	  PANEL_ACTION_NONE,
 	  NULL,
 	  NULL,
@@ -119,8 +120,8 @@ static PanelAddtoItemInfo special_addto_items [] = {
 
 	{ PANEL_ADDTO_LAUNCHER_MENU,
 	  N_("Application Launcher..."),
-	  N_("Launch a program that is already in the GNOME menu"),
-	  "launcher-program",
+	  N_("Copy a launcher from the applications menu"),
+	  PANEL_ICON_LAUNCHER,
 	  PANEL_ACTION_NONE,
 	  NULL,
 	  NULL,
@@ -135,7 +136,7 @@ static PanelAddtoItemInfo internal_addto_items [] = {
 	{ PANEL_ADDTO_MENU,
 	  N_("Main Menu"),
 	  N_("The main GNOME menu"),
-	  PANEL_MAIN_MENU_ICON,
+	  PANEL_ICON_MAIN_MENU,
 	  PANEL_ACTION_NONE,
 	  NULL,
 	  NULL,
@@ -146,7 +147,7 @@ static PanelAddtoItemInfo internal_addto_items [] = {
 	{ PANEL_ADDTO_MENUBAR,
 	  N_("Menu Bar"),
 	  N_("A custom menu bar"),
-	  PANEL_GNOME_LOGO_ICON,
+	  PANEL_ICON_MAIN_MENU,
 	  PANEL_ACTION_NONE,
 	  NULL,
 	  NULL,
@@ -157,7 +158,7 @@ static PanelAddtoItemInfo internal_addto_items [] = {
 	{ PANEL_ADDTO_SEPARATOR,
 	  N_("Separator"),
 	  N_("A separator to organize the panel items"),
-	  PANEL_SEPARATOR_ICON,
+	  PANEL_ICON_SEPARATOR,
 	  PANEL_ACTION_NONE,
 	  NULL,
 	  NULL,
@@ -168,7 +169,7 @@ static PanelAddtoItemInfo internal_addto_items [] = {
 	{ PANEL_ADDTO_DRAWER,
 	  N_("Drawer"),
 	  N_("A pop out drawer to store other items in"),
-	  PANEL_DRAWER_ICON,
+	  PANEL_ICON_DRAWER,
 	  PANEL_ACTION_NONE,
 	  NULL,
 	  NULL,
@@ -349,16 +350,19 @@ panel_addto_setup_drag (GtkTreeView          *tree_view,
 
 static void
 panel_addto_setup_launcher_drag (GtkTreeView *tree_view,
-				 const char  *uri)
+				 const char  *path)
 {
         static GtkTargetEntry target[] = {
 		{ "text/uri-list", 0, 0 }
 	};
+	char *uri;
 	char *uri_list;
 
+	uri = g_filename_to_uri (path, NULL, NULL);
 	uri_list = g_strconcat (uri, "\r\n", NULL);
 	panel_addto_setup_drag (tree_view, target, uri_list);
 	g_free (uri_list);
+	g_free (uri);
 }
 
 static void
@@ -511,6 +515,10 @@ panel_addto_append_special_applets (PanelAddtoDialog *dialog,
 			special_addto_items [i].name = _(special_addto_items [i].name);
 			special_addto_items [i].description = _(special_addto_items [i].description);
 		}
+
+		if (special_addto_items [i].type == PANEL_ADDTO_LAUNCHER_NEW
+		    && panel_lockdown_get_disable_command_line ())
+			continue;
 		
 		panel_addto_append_item (dialog, model, &special_addto_items [i]);
 	}
@@ -889,12 +897,6 @@ panel_addto_dialog_destroy (GtkWidget *widget_dialog,
 static void
 panel_addto_present_applications (PanelAddtoDialog *dialog)
 {
-	if (dialog->applet_search_text)
-		g_free (dialog->applet_search_text);
-
-	dialog->applet_search_text = g_strdup (gtk_entry_get_text (GTK_ENTRY (dialog->search_entry)));
-	gtk_entry_set_text (GTK_ENTRY (dialog->search_entry), "");
-
 	if (dialog->filter_application_model == NULL)
 		panel_addto_make_application_model (dialog);
 	gtk_tree_view_set_model (GTK_TREE_VIEW (dialog->tree_view),
@@ -902,6 +904,13 @@ panel_addto_present_applications (PanelAddtoDialog *dialog)
 	gtk_window_set_focus (GTK_WINDOW (dialog->addto_dialog),
 			      dialog->search_entry);
 	gtk_widget_set_sensitive (dialog->back_button, TRUE);
+
+	if (dialog->applet_search_text)
+		g_free (dialog->applet_search_text);
+
+	dialog->applet_search_text = g_strdup (gtk_entry_get_text (GTK_ENTRY (dialog->search_entry)));
+	/* show everything */
+	gtk_entry_set_text (GTK_ENTRY (dialog->search_entry), "");
 }
 
 static void
@@ -1114,8 +1123,12 @@ panel_addto_filter_func (GtkTreeModel *model,
 	if (data == NULL)
 		return FALSE;
 
-	//FIXME this doesn't really work for the GtkTreeStore used for
-	//the .desktop files
+	/* This is more a workaround than anything else: show all the root
+	 * items in a tree store */
+	if (GTK_IS_TREE_STORE (model) &&
+	    gtk_tree_store_iter_depth (GTK_TREE_STORE (model), iter) == 0)
+		return TRUE;
+
 	return (panel_util_utf8_strstrcase (data->name,
 					    dialog->search_text) != NULL ||
 	        panel_util_utf8_strstrcase (data->description,
